from array import array
from ctypes import *
from .api import api
from .const import *
from .library import library
from .oscilloscopechanneltrigger import OscilloscopeChannelTrigger


class OscilloscopeChannel(object):
    """"""

    def __init__(self, handle, ch):
        self._handle = handle
        self._ch = ch
        self._trigger = OscilloscopeChannelTrigger(handle, ch) if self.has_trigger else None

    def _get_trigger(self):
        return self._trigger

    def _get_is_available(self):
        """ Check whether the channel is available. """
        value = api.ScpChIsAvailable(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_connector_type(self):
        """ Channel connector type. """
        value = api.ScpChGetConnectorType(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_is_differential(self):
        """ Check whether the channel has a differential input. """
        value = api.ScpChIsDifferential(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_impedance(self):
        """ Channel input impedance. """
        value = api.ScpChGetImpedance(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_bandwidths(self):
        """ Supported input bandwidths. """
        count = api.ScpChGetBandwidths(self._handle, self._ch, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_double * count)()
        api.ScpChGetBandwidths(self._handle, self._ch, values, count)
        library.check_last_status_raise_on_error()
        return array('d', values)

    def _get_bandwidth(self):
        """ Current channel input bandwidth. """
        value = api.ScpChGetBandwidth(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_bandwidth(self, value):
        api.ScpChSetBandwidth(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_couplings(self):
        """ Supported coupling kinds. """
        value = api.ScpChGetCouplings(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_coupling(self):
        """ Currently set coupling. """
        value = api.ScpChGetCoupling(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_coupling(self, value):
        api.ScpChSetCoupling(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_enabled(self):
        """ Check whether a specified channel is currently enabled. """
        value = api.ScpChGetEnabled(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_enabled(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.ScpChSetEnabled(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_probe_gain(self):
        """ Currently set channel probe gain. """
        value = api.ScpChGetProbeGain(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_probe_gain(self, value):
        api.ScpChSetProbeGain(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_probe_offset(self):
        """ Currently set channel probe offset. """
        value = api.ScpChGetProbeOffset(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_probe_offset(self, value):
        api.ScpChSetProbeOffset(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_auto_ranging(self):
        """ Check whether auto ranging is enabled. """
        value = api.ScpChGetAutoRanging(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_auto_ranging(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.ScpChSetAutoRanging(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_ranges(self):
        """ Supported input ranges, with the currently selected coupling. """
        count = api.ScpChGetRanges(self._handle, self._ch, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_double * count)()
        api.ScpChGetRanges(self._handle, self._ch, values, count)
        library.check_last_status_raise_on_error()
        return array('d', values)

    def _get_range(self):
        """ Currently selected input range. """
        value = api.ScpChGetRange(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_range(self, value):
        api.ScpChSetRange(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_has_safe_ground(self):
        """ Check whether the specified channel has SafeGround. """
        value = api.ScpChHasSafeGround(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_safe_ground_enabled(self):
        """ Check whether SafeGround is enabled. """
        value = api.ScpChGetSafeGroundEnabled(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_safe_ground_enabled(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.ScpChSetSafeGroundEnabled(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_safe_ground_threshold_min(self):
        """ Minimum SafeGround threshold current. """
        value = api.ScpChGetSafeGroundThresholdMin(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_safe_ground_threshold_max(self):
        """ Maximum SafeGround threshold current. """
        value = api.ScpChGetSafeGroundThresholdMax(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_safe_ground_threshold(self):
        """ Actual SafeGround threshold current. """
        value = api.ScpChGetSafeGroundThreshold(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_safe_ground_threshold(self, value):
        api.ScpChSetSafeGroundThreshold(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def verify_safe_ground_threshold(self, threshold):
        """ Verify if the required threshold current can be set.

        :param threshold: The required threshold current, in Ampere.
        :returns: The SafeGround threshold current that would be set.
        .. versionadded:: 0.6
        """
        result = api.ScpChVerifySafeGroundThreshold(self._handle, self._ch, threshold)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_trigger(self):
        """ Check whether the specified channel has trigger support with the currently selected measure mode. """
        value = api.ScpChHasTrigger(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_data_value_min(self):
        """ Minimum value of the input range the current data was measured with. """
        value = api.ScpChGetDataValueMin(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_value_max(self):
        """ Maximum value of the input range the current data was measured with. """
        value = api.ScpChGetDataValueMax(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_raw_type(self):
        """ Get raw data type. """
        value = api.ScpChGetDataRawType(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_raw_value_min(self):
        """ Get possible raw data minimum value. """
        value = api.ScpChGetDataRawValueMin(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_raw_value_zero(self):
        """ Get raw data value which equals zero. """
        value = api.ScpChGetDataRawValueZero(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_raw_value_max(self):
        """ Get possible raw data maximum value. """
        value = api.ScpChGetDataRawValueMax(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_is_range_max_reachable(self):
        """ Check whether the ranges maximum is reachable. """
        value = api.ScpChIsRangeMaxReachable(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_has_connection_test(self):
        """ Check whether a specified channel supports connection testing. """
        value = api.ScpChHasConnectionTest(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    is_available = property(_get_is_available)
    connector_type = property(_get_connector_type)
    is_differential = property(_get_is_differential)
    impedance = property(_get_impedance)
    bandwidths = property(_get_bandwidths)
    bandwidth = property(_get_bandwidth, _set_bandwidth)
    couplings = property(_get_couplings)
    coupling = property(_get_coupling, _set_coupling)
    enabled = property(_get_enabled, _set_enabled)
    probe_gain = property(_get_probe_gain, _set_probe_gain)
    probe_offset = property(_get_probe_offset, _set_probe_offset)
    auto_ranging = property(_get_auto_ranging, _set_auto_ranging)
    ranges = property(_get_ranges)
    range = property(_get_range, _set_range)
    has_safe_ground = property(_get_has_safe_ground)
    safe_ground_enabled = property(_get_safe_ground_enabled, _set_safe_ground_enabled)
    safe_ground_threshold_min = property(_get_safe_ground_threshold_min)
    safe_ground_threshold_max = property(_get_safe_ground_threshold_max)
    safe_ground_threshold = property(_get_safe_ground_threshold, _set_safe_ground_threshold)
    has_trigger = property(_get_has_trigger)
    data_value_min = property(_get_data_value_min)
    data_value_max = property(_get_data_value_max)
    data_raw_type = property(_get_data_raw_type)
    data_raw_value_min = property(_get_data_raw_value_min)
    data_raw_value_zero = property(_get_data_raw_value_zero)
    data_raw_value_max = property(_get_data_raw_value_max)
    is_range_max_reachable = property(_get_is_range_max_reachable)
    has_connection_test = property(_get_has_connection_test)
    trigger = property(_get_trigger)



from .api import api
from .const import *
from .utils import *
from .library import library
from .object import Object
from .triggerinputs import TriggerInputs
from .triggeroutputs import TriggerOutputs


class Device(Object):
    """"""

    def __init__(self, handle):
        super(Device, self).__init__(handle)
        self._trigger_inputs = TriggerInputs(handle)
        self._trigger_outputs = TriggerOutputs(handle)

    def _get_trigger_inputs(self):
        return self._trigger_inputs

    def _get_trigger_outputs(self):
        return self._trigger_outputs

    def _get_driver_version(self):
        """ Version number of the driver used by the device. """
        value = api.DevGetDriverVersion(self._handle)
        library.check_last_status_raise_on_error()
        return convert_version(value)

    def _get_firmware_version(self):
        """ Version number of the firmware used by the device. """
        value = api.DevGetFirmwareVersion(self._handle)
        library.check_last_status_raise_on_error()
        return convert_version(value)

    def _get_calibration_date(self):
        """ Calibration date of the device. """
        value = api.DevGetCalibrationDate(self._handle)
        library.check_last_status_raise_on_error()
        return convert_date(value)

    def _get_calibration_token(self):
        """ Calibration token of the device. """
        length = api.DevGetCalibrationToken(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.DevGetCalibrationToken(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_serial_number(self):
        """ Serial number of the device. """
        value = api.DevGetSerialNumber(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_ipv4_address(self):
        """ IPv4 address of the device. """
        value = api.DevGetIPv4Address(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_ip_port(self):
        """ IP port number of the device. """
        value = api.DevGetIPPort(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_product_id(self):
        """ Product id of the device. """
        value = api.DevGetProductId(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_vendor_id(self):
        """ Vendor id of the device. """
        value = api.DevGetVendorId(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_type(self):
        """ Device type. """
        value = api.DevGetType(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_name(self):
        """ Full name of the device. """
        length = api.DevGetName(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.DevGetName(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_name_short(self):
        """ Short name of the device. """
        length = api.DevGetNameShort(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.DevGetNameShort(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_name_shortest(self):
        """ Short name of the device without model postfix. """
        length = api.DevGetNameShortest(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.DevGetNameShortest(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_has_battery(self):
        """ Check whether the device has a battery. """
        value = api.DevHasBattery(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_battery_charge(self):
        """ Get battery charge in percent. """
        value = api.DevGetBatteryCharge(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_battery_time_to_empty(self):
        """ Get battery time to empty in minutes. """
        value = api.DevGetBatteryTimeToEmpty(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_battery_time_to_full(self):
        """ Get battery time to full in minutes. """
        value = api.DevGetBatteryTimeToFull(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_is_battery_charger_connected(self):
        """ Check whether the battery charger is connected. """
        value = api.DevIsBatteryChargerConnected(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_battery_charging(self):
        """ Check whether the battery is charging. """
        value = api.DevIsBatteryCharging(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_battery_broken(self):
        """ Check whether the battery is broken. """
        value = api.DevIsBatteryBroken(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def set_callback_removed(self, callback, data):
        """ Set a callback function which is called when the device is removed.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.DevSetCallbackRemoved(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    if platform.system() == 'Linux':
        def set_event_removed(self, event):
            """ Set an event file descriptor which is set when the device is removed.

            :param event: An event file descriptor. Use <tt><0</tt> to disable.
            """
            api.DevSetEventRemoved(self._handle, event)
            library.check_last_status_raise_on_error()

    if platform.system() == 'Windows':
        def set_event_removed(self, event):
            """ Set an event object handle which is set when the device is removed.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.DevSetEventRemoved(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_message_removed(self, wnd, wparam, lparam):
            """ Set a window handle to which a #WM_LIBTIEPIE_DEV_REMOVED message is sent when the device is removed.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            :param wparam: Optional user value for the ``wParam`` parameter of the message.
            :param lparam: Optional user value for the ``lParam`` parameter of the message.
            """
            api.DevSetMessageRemoved(self._handle, wnd, wparam, lparam)
            library.check_last_status_raise_on_error()

    driver_version = property(_get_driver_version)
    firmware_version = property(_get_firmware_version)
    calibration_date = property(_get_calibration_date)
    calibration_token = property(_get_calibration_token)
    serial_number = property(_get_serial_number)
    ipv4_address = property(_get_ipv4_address)
    ip_port = property(_get_ip_port)
    product_id = property(_get_product_id)
    vendor_id = property(_get_vendor_id)
    type = property(_get_type)
    name = property(_get_name)
    name_short = property(_get_name_short)
    name_shortest = property(_get_name_shortest)
    has_battery = property(_get_has_battery)
    battery_charge = property(_get_battery_charge)
    battery_time_to_empty = property(_get_battery_time_to_empty)
    battery_time_to_full = property(_get_battery_time_to_full)
    is_battery_charger_connected = property(_get_is_battery_charger_connected)
    is_battery_charging = property(_get_is_battery_charging)
    is_battery_broken = property(_get_is_battery_broken)
    trigger_inputs = property(_get_trigger_inputs)
    trigger_outputs = property(_get_trigger_outputs)



from array import array
from ctypes import c_uint8, c_float
from .api import api
from .const import *
from .utils import *
from .library import library
from .device import Device
from .oscilloscopechannels import OscilloscopeChannels
from .exceptions import *


class Oscilloscope(Device):
    """"""

    def __init__(self, handle):
        super(Oscilloscope, self).__init__(handle)
        self._channels = OscilloscopeChannels(handle)

    def _get_channels(self):
        return self._channels

    def get_data(self, count=None, raw=False):
        """ Get the measurement data for enabled channels.

        :param count: Number of samples to read, defaults to all.
        :param raw: Get raw data.

        :returns: `list` of `array.array`'s with sample data.
        """
        if not self.is_data_ready:
            raise UnsuccessfulError()

        channel_count = len(self.channels)

        # Calculate valid data start/length:
        if self._measure_mode == MM_BLOCK:
            length = int(self._record_length - round(self._pre_sample_ratio * self._record_length) + self.valid_pre_sample_count)
            start = self._record_length - length
        else:
            length = self._record_length
            start = 0

        if (count is not None) and (count >= 0) and (count < length):
            length = count

        # Create pointer array:
        pointers = api.HlpPointerArrayNew(channel_count)

        try:
            # Allocate memory and fill pointer array:
            result = [None] * channel_count
            for i in range(channel_count):
                if self._active_channels[i]:
                    if raw:
                        raw_type = self.channels[i].data_raw_type
                        if raw_type == DATARAWTYPE_INT8:
                            result[i] = array('b', [0]) * length
                        elif raw_type == DATARAWTYPE_INT16:
                            result[i] = array('h', [0]) * length
                        elif raw_type == DATARAWTYPE_INT32:
                            result[i] = array('l', [0]) * length
                        elif raw_type == DATARAWTYPE_INT64:
                            result[i] = array('q', [0]) * length
                        elif raw_type == DATARAWTYPE_UINT8:
                            result[i] = array('B', [0]) * length
                        elif raw_type == DATARAWTYPE_UINT16:
                            result[i] = array('H', [0]) * length
                        elif raw_type == DATARAWTYPE_UINT32:
                            result[i] = array('L', [0]) * length
                        elif raw_type == DATARAWTYPE_UINT64:
                            result[i] = array('Q', [0]) * length
                        elif raw_type == DATARAWTYPE_FLOAT32:
                            result[i] = array('f', [0]) * length
                        elif raw_type == DATARAWTYPE_FLOAT64:
                            result[i] = array('d', [0]) * length
                        else:
                            raise UnsuccessfulError()
                    else:
                        result[i] = array('f', [0]) * length
                    api.HlpPointerArraySet(pointers, i, cast(result[i].buffer_info()[0], c_void_p))

            # Get the data:
            if raw:
                api.ScpGetDataRaw(self._handle, pointers, channel_count, start, length)
            else:
                api.ScpGetData(self._handle, pointers, channel_count, start, length)
            library.check_last_status_raise_on_error()
        finally:
            # Delete pointer array:
            api.HlpPointerArrayDelete(pointers)

        return result

    def _get_valid_pre_sample_count(self):
        """ Number of valid pre samples in the measurement. """
        value = api.ScpGetValidPreSampleCount(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def get_data_raw(self, buffers, channel_count, start_index, sample_count):
        """ Get raw measurement data.

        :param buffers: Pointer to buffer with pointers to buffer for channel data, pointer buffer may contain ``None`` pointers.
        :param channel_count: Number of pointers in pointer buffer.
        :param start_index: Position in record to start reading.
        :param sample_count: Number of samples to read.
        :returns: Number of samples read.
        """
        result = api.ScpGetDataRaw(self._handle, buffers, channel_count, start_index, sample_count)
        library.check_last_status_raise_on_error()
        return result

    def get_data_async_completed(self):
        """ Check whether the data download is completed.

        :returns: ``True`` if completed, ``False`` otherwise.
        """
        result = api.ScpIsGetDataAsyncCompleted(self._handle)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def start_get_data_async(self, buffers, channel_count, start_index, sample_count):
        """ Start the download of measurement data for specified channels.

        :param buffers: A pointer to a buffer with pointers to buffers for channel data, the pointer buffer may contain ``None`` pointers.
        :param channel_count: The number of pointers in the pointer buffer.
        :param start_index: The position in the record to start reading.
        :param sample_count: The number of samples to read.
        .. versionadded:: 0.6
        """
        api.ScpStartGetDataAsync(self._handle, buffers, channel_count, start_index, sample_count)
        library.check_last_status_raise_on_error()

    def start_get_data_async_raw(self, buffers, channel_count, start_index, sample_count):
        """ Start the download of raw measurement data for specified channels.

        :param buffers: Pointer to buffer with pointers to buffer for channel data, pointer buffer may contain ``None`` pointers.
        :param channel_count: Number of pointers in pointer buffer.
        :param start_index: Position in record to start reading.
        :param sample_count: Number of samples to read.
        .. versionadded:: 0.6
        """
        api.ScpStartGetDataAsyncRaw(self._handle, buffers, channel_count, start_index, sample_count)
        library.check_last_status_raise_on_error()

    def cancel_get_data_async(self):
        """ Cancel the download of measurement data.

        :returns: ``True`` if successful, ``False`` otherwise.
        .. versionadded:: 0.6
        """
        result = api.ScpCancelGetDataAsync(self._handle)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def set_callback_data_ready(self, callback, data):
        """ Set a callback function which is called when the oscilloscope has new measurement data ready.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.ScpSetCallbackDataReady(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    def set_callback_data_overflow(self, callback, data):
        """ Set a callback function which is called when the oscilloscope streaming measurement caused an data overflow.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.ScpSetCallbackDataOverflow(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    def set_callback_connection_test_completed(self, callback, data):
        """ Set a callback function which is called when the oscilloscope connection test is completed.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.ScpSetCallbackConnectionTestCompleted(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    def set_callback_triggered(self, callback, data):
        """ Set a callback function which is called when the oscilloscope is triggered.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.ScpSetCallbackTriggered(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    if platform.system() == 'Linux':
        def set_event_data_ready(self, event):
            """ Set an event file descriptor which is set when the oscilloscope has new measurement data ready.

            :param event: An event file descriptor. Use ``&lt;0`` to disable.
            """
            api.ScpSetEventDataReady(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_event_data_overflow(self, event):
            """ Set an event file descriptor which is set when the oscilloscope streaming measurement caused an data overflow.

            :param event: An event file descriptor. Use ``&lt;0`` to disable.
            """
            api.ScpSetEventDataOverflow(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_event_connection_test_completed(self, event):
            """ Set an event file descriptor which is set when the oscilloscope connection test is completed.

            :param event: An event file descriptor. Use ``&lt;0`` to disable.
            """
            api.ScpSetEventConnectionTestCompleted(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_event_triggered(self, event):
            """ Set an event file descriptor which is set when the oscilloscope is triggered.

            :param event: An event file descriptor. Use ``&lt;0`` to disable.
            """
            api.ScpSetEventTriggered(self._handle, event)
            library.check_last_status_raise_on_error()

    if platform.system() == 'Windows':
        def set_event_data_ready(self, event):
            """ Set an event object handle which is set when the oscilloscope has new measurement data ready.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.ScpSetEventDataReady(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_event_data_overflow(self, event):
            """ Set an event object handle which is set when the oscilloscope streaming measurement caused an data overflow.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.ScpSetEventDataOverflow(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_event_connection_test_completed(self, event):
            """ Set an event object handle which is set when the oscilloscope connection test is completed.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.ScpSetEventConnectionTestCompleted(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_event_triggered(self, event):
            """ Set an event object handle which is set when the oscilloscope is triggered.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.ScpSetEventTriggered(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_message_data_ready(self, wnd, wparam, lparam):
            """ Set a window handle to which a #WM_LIBTIEPIE_SCP_DATAREADY message is sent when the oscilloscope has new measurement data ready.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            :param wparam: Optional user value for the ``wParam`` parameter of the message.
            :param lparam: Optional user value for the ``lParam`` parameter of the message.
            """
            api.ScpSetMessageDataReady(self._handle, wnd, wparam, lparam)
            library.check_last_status_raise_on_error()

        def set_message_data_overflow(self, wnd, wparam, lparam):
            """ Set a window handle to which a #WM_LIBTIEPIE_SCP_DATAOVERFLOW message is sent when the oscilloscope streaming measurement caused an data overflow.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            :param wparam: Optional user value for the ``wParam`` parameter of the message.
            :param lparam: Optional user value for the ``lParam`` parameter of the message.
            """
            api.ScpSetMessageDataOverflow(self._handle, wnd, wparam, lparam)
            library.check_last_status_raise_on_error()

        def set_message_connection_test_completed(self, wnd, wparam, lparam):
            """ Set a window handle to which a #WM_LIBTIEPIE_SCP_CONNECTIONTESTCOMPLETED message is sent when the oscilloscope connection test is completed.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            :param wparam: Optional user value for the ``wParam`` parameter of the message.
            :param lparam: Optional user value for the ``lParam`` parameter of the message.
            """
            api.ScpSetMessageConnectionTestCompleted(self._handle, wnd, wparam, lparam)
            library.check_last_status_raise_on_error()

        def set_message_triggered(self, wnd, wparam, lparam):
            """ Set a window handle to which a #WM_LIBTIEPIE_SCP_TRIGGERED message is sent when the oscilloscope is triggered.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            :param wparam: Optional user value for the ``wParam`` parameter of the message.
            :param lparam: Optional user value for the ``lParam`` parameter of the message.
            """
            api.ScpSetMessageTriggered(self._handle, wnd, wparam, lparam)
            library.check_last_status_raise_on_error()

    def start(self):
        """ Start a single measurement. """
        if self.is_running:
            raise MeasurementRunningError()

        # Cache some values, needed for getting data:
        self._measure_mode = self.measure_mode
        self._record_length = self.record_length
        if self._measure_mode == MM_BLOCK:
            self._pre_sample_ratio = self.pre_sample_ratio
        self._active_channels = []
        for ch in self.channels:
            self._active_channels.append(ch.enabled)

        result = api.ScpStart(self._handle)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def stop(self):
        """ Stop a running measurement.

        """
        api.ScpStop(self._handle)
        library.check_last_status_raise_on_error()

    def force_trigger(self):
        """ Force a trigger.

        """
        api.ScpForceTrigger(self._handle)
        library.check_last_status_raise_on_error()

    def _get_measure_modes(self):
        """ Supported measure modes. """
        value = api.ScpGetMeasureModes(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_measure_mode(self):
        """ Current measure mode. """
        value = api.ScpGetMeasureMode(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_measure_mode(self, value):
        api.ScpSetMeasureMode(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_is_running(self):
        """ Check whether the oscilloscope is currently measuring. """
        value = api.ScpIsRunning(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_triggered(self):
        """ Check whether the oscilloscope has triggered. """
        value = api.ScpIsTriggered(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_time_out_triggered(self):
        """ Check whether the trigger was caused by the trigger time out. """
        value = api.ScpIsTimeOutTriggered(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_force_triggered(self):
        """ Check whether the trigger was caused by ScpForceTrigger. """
        value = api.ScpIsForceTriggered(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_data_ready(self):
        """ Check whether new, unread measured data is available. """
        value = api.ScpIsDataReady(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_data_overflow(self):
        """ Check whether a data overflow has occurred. """
        value = api.ScpIsDataOverflow(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_auto_resolution_modes(self):
        """ Supported auto resolution modes. """
        value = api.ScpGetAutoResolutionModes(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_auto_resolution_mode(self):
        """ Current auto resolution mode. """
        value = api.ScpGetAutoResolutionMode(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_auto_resolution_mode(self, value):
        api.ScpSetAutoResolutionMode(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_resolutions(self):
        """ :class:`array.array` of supported resolutions. """
        count = api.ScpGetResolutions(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_uint8 * count)()
        api.ScpGetResolutions(self._handle, values, count)
        library.check_last_status_raise_on_error()
        return array('B', values)

    def _get_resolution(self):
        """ Current resolution. """
        value = api.ScpGetResolution(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_resolution(self, value):
        api.ScpSetResolution(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_is_resolution_enhanced(self):
        """ Check whether the currently selected resolution is enhanced or a native resolution of the hardware. """
        value = api.ScpIsResolutionEnhanced(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_clock_sources(self):
        """ Supported clock sources. """
        value = api.ScpGetClockSources(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_clock_source(self):
        """ Currently selected clock source. """
        value = api.ScpGetClockSource(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_clock_source(self, value):
        api.ScpSetClockSource(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_clock_source_frequencies(self):
        """ :class:`array.array` of supported clock source frequencies. """
        count = api.ScpGetClockSourceFrequencies(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_double * count)()
        api.ScpGetClockSourceFrequencies(self._handle, values, count)
        library.check_last_status_raise_on_error()
        return array('d', values)

    def _get_clock_source_frequency(self):
        """ Current clock source frequency. """
        value = api.ScpGetClockSourceFrequency(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_clock_source_frequency(self, value):
        api.ScpSetClockSourceFrequency(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_clock_outputs(self):
        """ Supported clock outputs. """
        value = api.ScpGetClockOutputs(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_clock_output(self):
        """ Currently selected clock output. """
        value = api.ScpGetClockOutput(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_clock_output(self, value):
        api.ScpSetClockOutput(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_clock_output_frequencies(self):
        """ :class:`array.array` of supported clock output frequencies. """
        count = api.ScpGetClockOutputFrequencies(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_double * count)()
        api.ScpGetClockOutputFrequencies(self._handle, values, count)
        library.check_last_status_raise_on_error()
        return array('d', values)

    def _get_clock_output_frequency(self):
        """ Current clock output frequency. """
        value = api.ScpGetClockOutputFrequency(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_clock_output_frequency(self, value):
        api.ScpSetClockOutputFrequency(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_sample_frequency_max(self):
        """ Maximum supported sample frequency. """
        value = api.ScpGetSampleFrequencyMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_sample_frequency(self):
        """ Currently selected sample frequency. """
        value = api.ScpGetSampleFrequency(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_sample_frequency(self, value):
        api.ScpSetSampleFrequency(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_sample_frequency(self, sample_frequency):
        """ Verify if a required sample frequency can be set, without actually setting the hardware itself.

        :param sample_frequency: The required sample frequency, in Hz.
        :returns: The sample frequency that would have been set, if ScpSetSampleFrequency() was used.
        """
        result = api.ScpVerifySampleFrequency(self._handle, sample_frequency)
        library.check_last_status_raise_on_error()
        return result

    def _get_record_length_max(self):
        """ Maximum supported record length. """
        value = api.ScpGetRecordLengthMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_record_length(self):
        """ Currently selected record length. """
        value = api.ScpGetRecordLength(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_record_length(self, value):
        api.ScpSetRecordLength(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_record_length(self, record_length):
        """ Verify if a required record length can be set, without actually setting the hardware itself.

        :param record_length: The required record length, in samples.
        :returns: The record length that would have been set, if ScpSetRecordLength() was used.
        """
        result = api.ScpVerifyRecordLength(self._handle, record_length)
        library.check_last_status_raise_on_error()
        return result

    def _get_pre_sample_ratio(self):
        """ Current pre sample ratio. """
        value = api.ScpGetPreSampleRatio(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_pre_sample_ratio(self, value):
        api.ScpSetPreSampleRatio(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_segment_count_max(self):
        """ Maximum supported number of segments. """
        value = api.ScpGetSegmentCountMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_segment_count(self):
        """ Currently selected number of segments. """
        value = api.ScpGetSegmentCount(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_segment_count(self, value):
        api.ScpSetSegmentCount(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_segment_count(self, segment_count):
        """ Verify if a required number of segments can be set, without actually setting the hardware itself.

        :param segment_count: The required number of segments.
        :returns: The actually number of segments that would have been set, if ScpSetSegmentCount() was used.
        """
        result = api.ScpVerifySegmentCount(self._handle, segment_count)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_trigger(self):
        """ Check whether the oscilloscope has trigger support with the currently selected measure mode. """
        value = api.ScpHasTrigger(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_trigger_time_out(self):
        """ Currently selected trigger time out in seconds. """
        value = api.ScpGetTriggerTimeOut(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_trigger_time_out(self, value):
        api.ScpSetTriggerTimeOut(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_trigger_time_out(self, time_out):
        """ Verify if a required trigger time out can be set, without actually setting the hardware itself.

        :param time_out: The required trigger time out in seconds, or #TO_INFINITY.
        :returns: The trigger time out that would have been set, if ScpSetTriggerTimeOut() was used.
        """
        result = api.ScpVerifyTriggerTimeOut(self._handle, time_out)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_trigger_delay(self):
        """ Check whether the oscilloscope has trigger delay support with the currently selected measure mode. """
        value = api.ScpHasTriggerDelay(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_trigger_delay_max(self):
        """ Maximum trigger delay in seconds, for the currently selected measure mode and sample frequency. """
        value = api.ScpGetTriggerDelayMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_trigger_delay(self):
        """ Currently selected trigger delay in seconds. """
        value = api.ScpGetTriggerDelay(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_trigger_delay(self, value):
        api.ScpSetTriggerDelay(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_trigger_delay(self, delay):
        """ Verify if a required trigger delay can be set, without actually setting the hardware itself.

        :param delay: The required trigger delay in seconds.
        :returns: The trigger delay that would have been set, if ScpSetTriggerDelay() was used.
        """
        result = api.ScpVerifyTriggerDelay(self._handle, delay)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_trigger_hold_off(self):
        """ Check whether the oscilloscope has trigger hold off support with the currently selected measure mode. """
        value = api.ScpHasTriggerHoldOff(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_trigger_hold_off_count_max(self):
        """ Maximum trigger hold off count in samples. """
        value = api.ScpGetTriggerHoldOffCountMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_trigger_hold_off_count(self):
        """ Trigger hold off count in samples. """
        value = api.ScpGetTriggerHoldOffCount(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_trigger_hold_off_count(self, value):
        api.ScpSetTriggerHoldOffCount(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_has_connection_test(self):
        """ Check whether the specified oscilloscope supports connection testing. """
        value = api.ScpHasConnectionTest(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def start_connection_test(self):
        """ Perform a connection test on all enabled channels.

        """
        api.ScpStartConnectionTest(self._handle)
        library.check_last_status_raise_on_error()

    def _get_is_connection_test_completed(self):
        """ Check whether the connection test on a specified oscilloscope is completed. """
        value = api.ScpIsConnectionTestCompleted(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def get_connection_test_data(self):
        """ Get the connection test result data.

        :returns: :class:`list` of :class:`.TriState` values.
        """
        if not self.is_connection_test_completed:
            raise UnsuccessfulError()

        channel_count = len(self.channels)

        # Allocate memory:
        buffer = (c_uint8 * channel_count)()

        # Get the data:
        channel_count = api.ScpGetConnectionTestData(self._handle, buffer, channel_count)
        library.check_last_status_raise_on_error()

        # Create result array:
        result = []
        for i in range(channel_count):
            result.append(convert_tristate(buffer[i]))

        return result

    valid_pre_sample_count = property(_get_valid_pre_sample_count)
    measure_modes = property(_get_measure_modes)
    measure_mode = property(_get_measure_mode, _set_measure_mode)
    is_running = property(_get_is_running)
    is_triggered = property(_get_is_triggered)
    is_time_out_triggered = property(_get_is_time_out_triggered)
    is_force_triggered = property(_get_is_force_triggered)
    is_data_ready = property(_get_is_data_ready)
    is_data_overflow = property(_get_is_data_overflow)
    auto_resolution_modes = property(_get_auto_resolution_modes)
    auto_resolution_mode = property(_get_auto_resolution_mode, _set_auto_resolution_mode)
    resolutions = property(_get_resolutions)
    resolution = property(_get_resolution, _set_resolution)
    is_resolution_enhanced = property(_get_is_resolution_enhanced)
    clock_sources = property(_get_clock_sources)
    clock_source = property(_get_clock_source, _set_clock_source)
    clock_source_frequencies = property(_get_clock_source_frequencies)
    clock_source_frequency = property(_get_clock_source_frequency, _set_clock_source_frequency)
    clock_outputs = property(_get_clock_outputs)
    clock_output = property(_get_clock_output, _set_clock_output)
    clock_output_frequencies = property(_get_clock_output_frequencies)
    clock_output_frequency = property(_get_clock_output_frequency, _set_clock_output_frequency)
    sample_frequency_max = property(_get_sample_frequency_max)
    sample_frequency = property(_get_sample_frequency, _set_sample_frequency)
    record_length_max = property(_get_record_length_max)
    record_length = property(_get_record_length, _set_record_length)
    pre_sample_ratio = property(_get_pre_sample_ratio, _set_pre_sample_ratio)
    segment_count_max = property(_get_segment_count_max)
    segment_count = property(_get_segment_count, _set_segment_count)
    has_trigger = property(_get_has_trigger)
    trigger_time_out = property(_get_trigger_time_out, _set_trigger_time_out)
    has_trigger_delay = property(_get_has_trigger_delay)
    trigger_delay_max = property(_get_trigger_delay_max)
    trigger_delay = property(_get_trigger_delay, _set_trigger_delay)
    has_trigger_hold_off = property(_get_has_trigger_hold_off)
    trigger_hold_off_count_max = property(_get_trigger_hold_off_count_max)
    trigger_hold_off_count = property(_get_trigger_hold_off_count, _set_trigger_hold_off_count)
    has_connection_test = property(_get_has_connection_test)
    is_connection_test_completed = property(_get_is_connection_test_completed)
    channels = property(_get_channels)



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='python-libtiepie',
      version='0.9.5',
      description='Python bindings for LibTiePie',
      license='MIT',
      author='TiePie engineering',
      author_email='support@tiepie.nl',
      url='https://github.com/TiePie/python-libtiepie',
      packages=find_packages(include=['libtiepie', 'libtiepie.*']),
      package_data={'libtiepie': ['_platform/*/*/*.dll']},
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: MIT License',
          'Operating System :: Microsoft :: Windows :: Windows 7',
          'Operating System :: Microsoft :: Windows :: Windows 8',
          'Operating System :: Microsoft :: Windows :: Windows 8.1',
          'Operating System :: Microsoft :: Windows :: Windows 10',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Topic :: Scientific/Engineering',
          'Topic :: Software Development',
      ])



import platform


VERSION_MAJOR = 0
VERSION_MINOR = 9
VERSION_RELEASE = 0
VERSION_NUMBER = "0.9.0"
VERSION = "0.9.0"

REVISION = 13850

HANDLE_INVALID = 0

TPDEVICEHANDLE_INVALID = HANDLE_INVALID

INTERFACE_DEVICE = 0x0000000000000001
INTERFACE_OSCILLOSCOPE = 0x0000000000000002
INTERFACE_GENERATOR = 0x0000000000000004
INTERFACE_I2CHOST = 0x0000000000000008
INTERFACE_SERVER = 0x0000000000000010

DEVICETYPE_OSCILLOSCOPE = 0x00000001
DEVICETYPE_GENERATOR = 0x00000002
DEVICETYPE_I2CHOST = 0x00000004
DEVICETYPE_COUNT = 3

IDKIND_PRODUCTID = 0x00000001
IDKIND_INDEX = 0x00000002
IDKIND_SERIALNUMBER = 0x00000004
IDKIND_COUNT = 3

STATUS_SUCCESS = 0
STATUS_VALUE_CLIPPED = 1
STATUS_VALUE_MODIFIED = 2
STATUS_UNSUCCESSFUL = -1
STATUS_NOT_SUPPORTED = -2
STATUS_INVALID_HANDLE = -3
STATUS_INVALID_VALUE = -4
STATUS_INVALID_CHANNEL = -5
STATUS_INVALID_TRIGGER_SOURCE = -6
STATUS_INVALID_DEVICE_TYPE = -7
STATUS_INVALID_DEVICE_INDEX = -8
STATUS_INVALID_PRODUCT_ID = -9
STATUS_INVALID_DEVICE_SERIALNUMBER = -10
STATUS_OBJECT_GONE = -11
STATUS_DEVICE_GONE = STATUS_OBJECT_GONE
STATUS_INTERNAL_ADDRESS = -12
STATUS_NOT_CONTROLLABLE = -13
STATUS_BIT_ERROR = -14
STATUS_NO_ACKNOWLEDGE = -15
STATUS_INVALID_CONTAINED_DEVICE_SERIALNUMBER = -16
STATUS_INVALID_INPUT = -17
STATUS_INVALID_OUTPUT = -18
STATUS_INVALID_DRIVER = -19
STATUS_NOT_AVAILABLE = -20
STATUS_INVALID_FIRMWARE = -21
STATUS_INVALID_INDEX = -22
STATUS_INVALID_EEPROM = -23
STATUS_INITIALIZATION_FAILED = -24
STATUS_LIBRARY_NOT_INITIALIZED = -25
STATUS_NO_TRIGGER_ENABLED = -26
STATUS_SYNCHRONIZATION_FAILED = -29
STATUS_INVALID_HS56_COMBINED_DEVICE = -30
STATUS_MEASUREMENT_RUNNING = -31
STATUS_INITIALIZATION_ERROR_10001 = -10001
STATUS_INITIALIZATION_ERROR_10002 = -10002
STATUS_INITIALIZATION_ERROR_10003 = -10003
STATUS_INITIALIZATION_ERROR_10004 = -10004
STATUS_INITIALIZATION_ERROR_10005 = -10005
STATUS_INITIALIZATION_ERROR_10006 = -10006

CONNECTORTYPE_UNKNOWN = 0x00000000
CONNECTORTYPE_BNC = 0x00000001
CONNECTORTYPE_BANANA = 0x00000002
CONNECTORTYPE_POWERPLUG = 0x00000004
CONNECTORTYPE_COUNT = 3
CONNECTORTYPE_MASK = (CONNECTORTYPE_BNC | CONNECTORTYPE_BANANA | CONNECTORTYPE_POWERPLUG)

DATARAWTYPE_UNKNOWN = 0x00000000
DATARAWTYPE_INT8 = 0x00000001
DATARAWTYPE_INT16 = 0x00000002
DATARAWTYPE_INT32 = 0x00000004
DATARAWTYPE_INT64 = 0x00000008
DATARAWTYPE_UINT8 = 0x00000010
DATARAWTYPE_UINT16 = 0x00000020
DATARAWTYPE_UINT32 = 0x00000040
DATARAWTYPE_UINT64 = 0x00000080
DATARAWTYPE_FLOAT32 = 0x00000100
DATARAWTYPE_FLOAT64 = 0x00000200
DATARAWTYPE_COUNT = 10
DATARAWTYPE_MASK_INT = (DATARAWTYPE_INT8 | DATARAWTYPE_INT16 | DATARAWTYPE_INT32 | DATARAWTYPE_INT64)
DATARAWTYPE_MASK_UINT = (DATARAWTYPE_UINT8 | DATARAWTYPE_UINT16 | DATARAWTYPE_UINT32 | DATARAWTYPE_UINT64)
DATARAWTYPE_MASK_FLOAT = (DATARAWTYPE_FLOAT32 | DATARAWTYPE_FLOAT64)
DATARAWTYPE_MASK_FIXED = (DATARAWTYPE_MASK_INT | DATARAWTYPE_MASK_UINT)

BOOL8_FALSE = 0
BOOL8_TRUE = 1

TRISTATE_UNDEFINED = 0
TRISTATE_FALSE = 1
TRISTATE_TRUE = 2

TRIGGERIO_INDEX_INVALID = 0xffff

STRING_LENGTH_NULL_TERMINATED = 0xffffffff

SERVER_STATUS_DISCONNECTED = 0
SERVER_STATUS_CONNECTING = 1
SERVER_STATUS_CONNECTED = 2
SERVER_STATUS_DISCONNECTING = 3
SERVER_ERROR_NONE = 0
SERVER_ERROR_UNKNOWN = 1
SERVER_ERROR_CONNECTIONREFUSED = 2
SERVER_ERROR_NETWORKUNREACHABLE = 3
SERVER_ERROR_TIMEDOUT = 4
SERVER_ERROR_HOSTNAMELOOKUPFAILED = 5

RANGEINDEX_AUTO = 0xffffffff

POINTER_ARRAY_MAX_LENGTH = 256

ARN_COUNT = 3

ARB_DISABLED = 0
ARB_NATIVEONLY = 1
ARB_ALL = 2

AR_UNKNOWN = 0
AR_DISABLED = (1 << ARB_DISABLED)
AR_NATIVEONLY = (1 << ARB_NATIVEONLY)
AR_ALL = (1 << ARB_ALL)

ARM_NONE = 0
ARM_ALL = ((1 << ARN_COUNT) - 1)
ARM_ENABLED = (ARM_ALL & ~AR_DISABLED)

CKN_COUNT = 5

CKB_DCV = 0
CKB_ACV = 1
CKB_DCA = 2
CKB_ACA = 3
CKB_OHM = 4

CK_UNKNOWN = 0
CK_DCV = (1 << CKB_DCV)
CK_ACV = (1 << CKB_ACV)
CK_DCA = (1 << CKB_DCA)
CK_ACA = (1 << CKB_ACA)
CK_OHM = (1 << CKB_OHM)

CKM_NONE = 0
CKM_V = (CK_DCV | CK_ACV)
CKM_A = (CK_DCA | CK_ACA)
CKM_OHM = (CK_OHM)
CKM_ASYMMETRICRANGE = (CKM_OHM)
CKM_SYMMETRICRANGE = (CKM_V | CKM_A)

CON_COUNT = 3

COB_DISABLED = 0
COB_SAMPLE = 1
COB_FIXED = 2

CO_DISABLED = (1 << COB_DISABLED)
CO_SAMPLE = (1 << COB_SAMPLE)
CO_FIXED = (1 << COB_FIXED)

COM_NONE = 0
COM_ALL = ((1 << CON_COUNT) - 1)
COM_ENABLED = (COM_ALL & ~CO_DISABLED)
COM_FREQUENCY = (CO_FIXED)

CSN_COUNT = 2

CSB_EXTERNAL = 0
CSB_INTERNAL = 1

CS_EXTERNAL = (1 << CSB_EXTERNAL)
CS_INTERNAL = (1 << CSB_INTERNAL)

CSM_NONE = 0
CSM_ALL = ((1 << CSN_COUNT) - 1)
CSM_FREQUENCY = (CS_EXTERNAL)

FMN_COUNT = 2

FMB_SIGNALFREQUENCY = 0
FMB_SAMPLEFREQUENCY = 1

FM_UNKNOWN = 0x00000000
FM_SIGNALFREQUENCY = (1 << FMB_SIGNALFREQUENCY)
FM_SAMPLEFREQUENCY = (1 << FMB_SAMPLEFREQUENCY)

FMM_NONE = 0x00000000
FMM_ALL = ((1 << FMN_COUNT) - 1)

GMN_COUNT = 12

GMB_CONTINUOUS = 0
GMB_BURST_COUNT = 1
GMB_GATED_PERIODS = 2
GMB_GATED = 3
GMB_GATED_PERIOD_START = 4
GMB_GATED_PERIOD_FINISH = 5
GMB_GATED_RUN = 6
GMB_GATED_RUN_OUTPUT = 7
GMB_BURST_SAMPLE_COUNT = 8
GMB_BURST_SAMPLE_COUNT_OUTPUT = 9
GMB_BURST_SEGMENT_COUNT = 10
GMB_BURST_SEGMENT_COUNT_OUTPUT = 11

GM_UNKNOWN = 0
GM_CONTINUOUS = (1 << GMB_CONTINUOUS)
GM_BURST_COUNT = (1 << GMB_BURST_COUNT)
GM_GATED_PERIODS = (1 << GMB_GATED_PERIODS)
GM_GATED = (1 << GMB_GATED)
GM_GATED_PERIOD_START = (1 << GMB_GATED_PERIOD_START)
GM_GATED_PERIOD_FINISH = (1 << GMB_GATED_PERIOD_FINISH)
GM_GATED_RUN = (1 << GMB_GATED_RUN)
GM_GATED_RUN_OUTPUT = (1 << GMB_GATED_RUN_OUTPUT)
GM_BURST_SAMPLE_COUNT = (1 << GMB_BURST_SAMPLE_COUNT)
GM_BURST_SAMPLE_COUNT_OUTPUT = (1 << GMB_BURST_SAMPLE_COUNT_OUTPUT)
GM_BURST_SEGMENT_COUNT = (1 << GMB_BURST_SEGMENT_COUNT)
GM_BURST_SEGMENT_COUNT_OUTPUT = (1 << GMB_BURST_SEGMENT_COUNT_OUTPUT)

GMM_NONE = 0
GMM_BURST_COUNT = (GM_BURST_COUNT)
GMM_GATED = (GM_GATED_PERIODS | GM_GATED | GM_GATED_PERIOD_START | GM_GATED_PERIOD_FINISH | GM_GATED_RUN | GM_GATED_RUN_OUTPUT)
GMM_BURST_SAMPLE_COUNT = (GM_BURST_SAMPLE_COUNT | GM_BURST_SAMPLE_COUNT_OUTPUT)
GMM_BURST_SEGMENT_COUNT = (GM_BURST_SEGMENT_COUNT | GM_BURST_SEGMENT_COUNT_OUTPUT)
GMM_BURST = (GMM_BURST_COUNT | GMM_BURST_SAMPLE_COUNT | GMM_BURST_SEGMENT_COUNT)
GMM_REQUIRE_TRIGGER = (GMM_GATED | GMM_BURST_SAMPLE_COUNT | GMM_BURST_SEGMENT_COUNT)
GMM_ALL = ((1 << GMN_COUNT) - 1)
GMM_SIGNALFREQUENCY = (GMM_ALL & ~GMM_BURST_SAMPLE_COUNT)
GMM_SAMPLEFREQUENCY = (GMM_ALL)
GMM_SINE = (GMM_SIGNALFREQUENCY)
GMM_TRIANGLE = (GMM_SIGNALFREQUENCY)
GMM_SQUARE = (GMM_SIGNALFREQUENCY)
GMM_DC = (GM_CONTINUOUS)
GMM_NOISE = (GM_CONTINUOUS | GM_GATED)
GMM_ARBITRARY = (GMM_SIGNALFREQUENCY | GMM_SAMPLEFREQUENCY)
GMM_PULSE = (GMM_SIGNALFREQUENCY & ~GMM_BURST_SEGMENT_COUNT)

GSN_COUNT = 4

GSB_STOPPED = 0
GSB_RUNNING = 1
GSB_BURSTACTIVE = 2
GSB_WAITING = 3

GS_STOPPED = (1 << GSB_STOPPED)
GS_RUNNING = (1 << GSB_RUNNING)
GS_BURSTACTIVE = (1 << GSB_BURSTACTIVE)
GS_WAITING = (1 << GSB_WAITING)

GSM_NONE = 0
GSM_ALL = ((1 << GSN_COUNT) - 1)

MMN_COUNT = 2

MMB_STREAM = 0
MMB_BLOCK = 1

MMM_NONE = 0
MMM_ALL = ((1 << MMN_COUNT) - 1)

MM_UNKNOWN = 0
MM_STREAM = (1 << MMB_STREAM)
MM_BLOCK = (1 << MMB_BLOCK)

STN_COUNT = 7

STB_SINE = 0
STB_TRIANGLE = 1
STB_SQUARE = 2
STB_DC = 3
STB_NOISE = 4
STB_ARBITRARY = 5
STB_PULSE = 6

ST_UNKNOWN = 0
ST_SINE = (1 << STB_SINE)
ST_TRIANGLE = (1 << STB_TRIANGLE)
ST_SQUARE = (1 << STB_SQUARE)
ST_DC = (1 << STB_DC)
ST_NOISE = (1 << STB_NOISE)
ST_ARBITRARY = (1 << STB_ARBITRARY)
ST_PULSE = (1 << STB_PULSE)

STM_NONE = 0
STM_AMPLITUDE = (ST_SINE | ST_TRIANGLE | ST_SQUARE | ST_NOISE | ST_ARBITRARY | ST_PULSE)
STM_OFFSET = (ST_SINE | ST_TRIANGLE | ST_SQUARE | ST_DC | ST_NOISE | ST_ARBITRARY | ST_PULSE)
STM_FREQUENCY = (ST_SINE | ST_TRIANGLE | ST_SQUARE | ST_NOISE | ST_ARBITRARY | ST_PULSE)
STM_PHASE = (ST_SINE | ST_TRIANGLE | ST_SQUARE | ST_ARBITRARY | ST_PULSE)
STM_SYMMETRY = (ST_SINE | ST_TRIANGLE | ST_SQUARE)
STM_WIDTH = (ST_PULSE)
STM_LEADINGEDGETIME = (ST_PULSE)
STM_TRAILINGEDGETIME = (ST_PULSE)
STM_DATALENGTH = (ST_ARBITRARY)
STM_DATA = (ST_ARBITRARY)
STM_EDGETIME = (STM_LEADINGEDGETIME & STM_TRAILINGEDGETIME)

TCN_COUNT = 5

TCB_NONE = 0
TCB_SMALLER = 1
TCB_LARGER = 2
TCB_INSIDE = 3
TCB_OUTSIDE = 4

TC_UNKNOWN = 0
TC_NONE = (1 << TCB_NONE)
TC_SMALLER = (1 << TCB_SMALLER)
TC_LARGER = (1 << TCB_LARGER)
TC_INSIDE = (1 << TCB_INSIDE)
TC_OUTSIDE = (1 << TCB_OUTSIDE)

TCM_NONE = 0
TCM_ALL = ((1 << TCN_COUNT) - 1)
TCM_ENABLED = (TCM_ALL & ~TC_NONE)

TH_ALLPRESAMPLES = 0xffffffffffffffff

DN_MAIN = 0
DN_SUB_FIRST = 1
DN_SUB_SECOND = 2

PGID_OSCILLOSCOPE = 1
PGID_GENERATOR = 2
PGID_EXTERNAL_DSUB = 3

SGID_MAIN = 0
SGID_CHANNEL1 = 1
SGID_CHANNEL2 = 2
SGID_PIN1 = 1
SGID_PIN2 = 2
SGID_PIN3 = 3

FID_SCP_TRIGGERED = 0
FID_GEN_START = 0
FID_GEN_STOP = 1
FID_GEN_NEW_PERIOD = 2
FID_EXT_TRIGGERED = 0

TIOID_SHIFT_PGID = 20
TIOID_SHIFT_DN = 24
TIOID_SHIFT_SGID = 8
TIOID_SHIFT_FID = 0

TIID_INVALID = 0
TIID_EXT1 = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_EXTERNAL_DSUB) << TIOID_SHIFT_PGID) | ((SGID_PIN1) << TIOID_SHIFT_SGID) | ((FID_EXT_TRIGGERED) << TIOID_SHIFT_FID))
TIID_EXT2 = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_EXTERNAL_DSUB) << TIOID_SHIFT_PGID) | ((SGID_PIN2) << TIOID_SHIFT_SGID) | ((FID_EXT_TRIGGERED) << TIOID_SHIFT_FID))
TIID_EXT3 = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_EXTERNAL_DSUB) << TIOID_SHIFT_PGID) | ((SGID_PIN3) << TIOID_SHIFT_SGID) | ((FID_EXT_TRIGGERED) << TIOID_SHIFT_FID))
TIID_GENERATOR_START = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_GENERATOR) << TIOID_SHIFT_PGID) | ((SGID_MAIN) << TIOID_SHIFT_SGID) | ((FID_GEN_START) << TIOID_SHIFT_FID))
TIID_GENERATOR_STOP = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_GENERATOR) << TIOID_SHIFT_PGID) | ((SGID_MAIN) << TIOID_SHIFT_SGID) | ((FID_GEN_STOP) << TIOID_SHIFT_FID))
TIID_GENERATOR_NEW_PERIOD = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_GENERATOR) << TIOID_SHIFT_PGID) | ((SGID_MAIN) << TIOID_SHIFT_SGID) | ((FID_GEN_NEW_PERIOD) << TIOID_SHIFT_FID))

TOID_INVALID = 0
TOID_EXT1 = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_EXTERNAL_DSUB) << TIOID_SHIFT_PGID) | ((SGID_PIN1) << TIOID_SHIFT_SGID) | ((FID_EXT_TRIGGERED) << TIOID_SHIFT_FID))
TOID_EXT2 = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_EXTERNAL_DSUB) << TIOID_SHIFT_PGID) | ((SGID_PIN2) << TIOID_SHIFT_SGID) | ((FID_EXT_TRIGGERED) << TIOID_SHIFT_FID))
TOID_EXT3 = ((DN_MAIN << TIOID_SHIFT_DN) | ((PGID_EXTERNAL_DSUB) << TIOID_SHIFT_PGID) | ((SGID_PIN3) << TIOID_SHIFT_SGID) | ((FID_EXT_TRIGGERED) << TIOID_SHIFT_FID))

TKN_COUNT = 15

TKB_RISINGEDGE = 0
TKB_FALLINGEDGE = 1
TKB_INWINDOW = 2
TKB_OUTWINDOW = 3
TKB_ANYEDGE = 4
TKB_ENTERWINDOW = 5
TKB_EXITWINDOW = 6
TKB_PULSEWIDTHPOSITIVE = 7
TKB_PULSEWIDTHNEGATIVE = 8
TKB_PULSEWIDTHEITHER = 9
TKB_RUNTPULSEPOSITIVE = 10
TKB_RUNTPULSENEGATIVE = 11
TKB_RUNTPULSEEITHER = 12
TKB_INTERVALRISING = 13
TKB_INTERVALFALLING = 14

TK_UNKNOWN = 0
TK_RISINGEDGE = (1 << TKB_RISINGEDGE)
TK_FALLINGEDGE = (1 << TKB_FALLINGEDGE)
TK_INWINDOW = (1 << TKB_INWINDOW)
TK_OUTWINDOW = (1 << TKB_OUTWINDOW)
TK_ANYEDGE = (1 << TKB_ANYEDGE)
TK_ENTERWINDOW = (1 << TKB_ENTERWINDOW)
TK_EXITWINDOW = (1 << TKB_EXITWINDOW)
TK_PULSEWIDTHPOSITIVE = (1 << TKB_PULSEWIDTHPOSITIVE)
TK_PULSEWIDTHNEGATIVE = (1 << TKB_PULSEWIDTHNEGATIVE)
TK_PULSEWIDTHEITHER = (1 << TKB_PULSEWIDTHEITHER)
TK_RUNTPULSEPOSITIVE = (1 << TKB_RUNTPULSEPOSITIVE)
TK_RUNTPULSENEGATIVE = (1 << TKB_RUNTPULSENEGATIVE)
TK_RUNTPULSEEITHER = (1 << TKB_RUNTPULSEEITHER)
TK_INTERVALRISING = (1 << TKB_INTERVALRISING)
TK_INTERVALFALLING = (1 << TKB_INTERVALFALLING)

TKM_NONE = 0
TKM_EDGE = (TK_RISINGEDGE | TK_FALLINGEDGE | TK_ANYEDGE)
TKM_WINDOW = (TK_INWINDOW | TK_OUTWINDOW | TK_ENTERWINDOW | TK_EXITWINDOW)
TKM_PULSEWIDTH = (TK_PULSEWIDTHPOSITIVE | TK_PULSEWIDTHNEGATIVE | TK_PULSEWIDTHEITHER)
TKM_RUNTPULSE = (TK_RUNTPULSEPOSITIVE | TK_RUNTPULSENEGATIVE | TK_RUNTPULSEEITHER)
TKM_PULSE = (TKM_PULSEWIDTH | TKM_RUNTPULSE)
TKM_INTERVAL = (TK_INTERVALRISING | TK_INTERVALFALLING)
TKM_TIME = (TKM_PULSEWIDTH | TKM_WINDOW | TKM_INTERVAL)
TKM_ALL = ((1 << TKN_COUNT) - 1)

TLMN_COUNT = 2

TLMB_RELATIVE = 0
TLMB_ABSOLUTE = 1

TLM_UNKNOWN = 0
TLM_RELATIVE = (1 << TLMB_RELATIVE)
TLM_ABSOLUTE = (1 << TLMB_ABSOLUTE)

TLMM_NONE = 0
TLMM_ALL = ((1 << TLMN_COUNT) - 1)

TO_INFINITY = -1

TOEN_COUNT = 6

TOEB_GENERATOR_START = 0
TOEB_GENERATOR_STOP = 1
TOEB_GENERATOR_NEWPERIOD = 2
TOEB_OSCILLOSCOPE_RUNNING = 3
TOEB_OSCILLOSCOPE_TRIGGERED = 4
TOEB_MANUAL = 5

TOE_UNKNOWN = 0
TOE_GENERATOR_START = (1 << TOEB_GENERATOR_START)
TOE_GENERATOR_STOP = (1 << TOEB_GENERATOR_STOP)
TOE_GENERATOR_NEWPERIOD = (1 << TOEB_GENERATOR_NEWPERIOD)
TOE_OSCILLOSCOPE_RUNNING = (1 << TOEB_OSCILLOSCOPE_RUNNING)
TOE_OSCILLOSCOPE_TRIGGERED = (1 << TOEB_OSCILLOSCOPE_TRIGGERED)
TOE_MANUAL = (1 << TOEB_MANUAL)

TOEM_NONE = 0
TOEM_GENERATOR = (TOE_GENERATOR_START | TOE_GENERATOR_STOP | TOE_GENERATOR_NEWPERIOD)
TOEM_OSCILLOSCOPE = (TOE_OSCILLOSCOPE_RUNNING | TOE_OSCILLOSCOPE_TRIGGERED)
TOEM_ALL = ((1 << TOEN_COUNT) - 1)

PID_NONE = 0
PID_COMBI = 2
PID_HS3 = 13
PID_HS4 = 15
PID_HP3 = 18
PID_TP450 = 19
PID_HS4D = 20
PID_HS5 = 22
PID_HS6D = 25
PID_ATS610004D = 31
PID_ATS605004D = 32
PID_WS6 = 34
PID_WS5 = 35

EVENTID_INVALID = 0
EVENTID_OBJ_REMOVED = 1
EVENTID_SCP_DATAREADY = 2
EVENTID_SCP_DATAOVERFLOW = 3
EVENTID_SCP_CONNECTIONTESTCOMPLETED = 4
EVENTID_SCP_TRIGGERED = 5
EVENTID_GEN_BURSTCOMPLETED = 6
EVENTID_GEN_CONTROLLABLECHANGED = 7
EVENTID_SRV_STATUSCHANGED = 8
EVENTID_SCP_SAFEGROUNDERROR = 9
EVENTID_SCP_GETDATAASYNCCOMPLETED = 10
EVENTID_DEV_BATTERYSTATUSCHANGED = 11

if platform.system() == 'Windows':
    WM_USER = 0x0400
    WM_LIBTIEPIE = (WM_USER + 1337)
    WM_LIBTIEPIE_LST_DEVICEADDED = (WM_LIBTIEPIE + 2)
    WM_LIBTIEPIE_LST_DEVICEREMOVED = (WM_LIBTIEPIE + 3)
    WM_LIBTIEPIE_LST_DEVICECANOPENCHANGED = (WM_LIBTIEPIE + 9)
    WM_LIBTIEPIE_NETSRV_ADDED = (WM_LIBTIEPIE + 11)
    WM_LIBTIEPIE_DEV_REMOVED = (WM_LIBTIEPIE + 4)
    WM_LIBTIEPIE_SCP_DATAREADY = (WM_LIBTIEPIE + 0)
    WM_LIBTIEPIE_SCP_DATAOVERFLOW = (WM_LIBTIEPIE + 1)
    WM_LIBTIEPIE_SCP_CONNECTIONTESTCOMPLETED = (WM_LIBTIEPIE + 7)
    WM_LIBTIEPIE_SCP_TRIGGERED = (WM_LIBTIEPIE + 8)
    WM_LIBTIEPIE_GEN_BURSTCOMPLETED = (WM_LIBTIEPIE + 5)
    WM_LIBTIEPIE_GEN_CONTROLLABLECHANGED = (WM_LIBTIEPIE + 6)
    WM_LIBTIEPIE_EVENT = (WM_LIBTIEPIE + 10)

AUTO_RESOLUTION_MODES = {
    AR_UNKNOWN: 'Unknown',
    AR_DISABLED: 'Disabled',
    AR_NATIVEONLY: 'Native only',
    AR_ALL: 'All',
}

COUPLINGS = {
    CK_UNKNOWN: 'Unknown',
    CK_DCV: 'DCV',
    CK_ACV: 'ACV',
    CK_DCA: 'DCA',
    CK_ACA: 'ACA',
    CK_OHM: 'Ohm',
}

CLOCK_OUTPUTS = {
    CO_DISABLED: 'Disabled',
    CO_SAMPLE: 'Sample',
    CO_FIXED: 'Fixed',
}

CLOCK_SOURCES = {
    CS_EXTERNAL: 'External',
    CS_INTERNAL: 'Internal',
}

CONNECTOR_TYPES = {
    CONNECTORTYPE_UNKNOWN: 'Unknown',
    CONNECTORTYPE_BNC: 'BNC',
    CONNECTORTYPE_BANANA: 'Banana',
    CONNECTORTYPE_POWERPLUG: 'Power plug',
}

DEVICE_TYPES = {
    DEVICETYPE_OSCILLOSCOPE: 'Oscilloscope',
    DEVICETYPE_GENERATOR: 'Generator',
    DEVICETYPE_I2CHOST: 'I2CHost',
}

FREQUENCY_MODES = {
    FM_UNKNOWN: 'Unknown',
    FM_SIGNALFREQUENCY: 'Signal frequency',
    FM_SAMPLEFREQUENCY: 'Sample frequency',
}

GENERATOR_MODES = {
    GM_UNKNOWN: 'Unknown',
    GM_CONTINUOUS: 'Continuous',
    GM_BURST_COUNT: 'Burst count',
    GM_GATED_PERIODS: 'Gated periods',
    GM_GATED: 'Gated',
    GM_GATED_PERIOD_START: 'Gated period start',
    GM_GATED_PERIOD_FINISH: 'Gated period finish',
    GM_GATED_RUN: 'Gated run',
    GM_GATED_RUN_OUTPUT: 'Gated run output',
    GM_BURST_SAMPLE_COUNT: 'Burst sample count',
    GM_BURST_SAMPLE_COUNT_OUTPUT: 'Burst sample count output',
    GM_BURST_SEGMENT_COUNT: 'Burst segment count',
    GM_BURST_SEGMENT_COUNT_OUTPUT: 'Burst segment count output',
}

GENERATOR_STATUSS = {
    GS_STOPPED: 'Stopped',
    GS_RUNNING: 'Running',
    GS_BURSTACTIVE: 'Burst active',
    GS_WAITING: 'Waiting',
}

INTERFACES = {
    INTERFACE_DEVICE: 'Device',
    INTERFACE_OSCILLOSCOPE: 'Oscilloscope',
    INTERFACE_GENERATOR: 'Generator',
    INTERFACE_I2CHOST: 'I2CHost',
    INTERFACE_SERVER: 'Server',
}

MEASURE_MODES = {
    MM_UNKNOWN: 'Unknown',
    MM_STREAM: 'Stream',
    MM_BLOCK: 'Block',
}

SERVER_ERRORS = {
    SERVER_ERROR_NONE: 'None',
    SERVER_ERROR_UNKNOWN: 'Unknown',
    SERVER_ERROR_CONNECTIONREFUSED: 'Connection refused',
    SERVER_ERROR_NETWORKUNREACHABLE: 'Network unreachable',
    SERVER_ERROR_TIMEDOUT: 'Timed out',
    SERVER_ERROR_HOSTNAMELOOKUPFAILED: 'Hostname lookup failed',
}

SERVER_STATUSS = {
    SERVER_STATUS_DISCONNECTED: 'Disconnected',
    SERVER_STATUS_CONNECTING: 'Connecting',
    SERVER_STATUS_CONNECTED: 'Connected',
    SERVER_STATUS_DISCONNECTING: 'Disconnecting',
}

SIGNAL_TYPES = {
    ST_UNKNOWN: 'Unknown',
    ST_SINE: 'Sine',
    ST_TRIANGLE: 'Triangle',
    ST_SQUARE: 'Square',
    ST_DC: 'DC',
    ST_NOISE: 'Noise',
    ST_ARBITRARY: 'Arbitrary',
    ST_PULSE: 'Pulse',
}

TRIGGER_CONDITIONS = {
    TC_UNKNOWN: 'Unknown',
    TC_NONE: 'None',
    TC_SMALLER: 'Smaller',
    TC_LARGER: 'Larger',
    TC_INSIDE: 'Inside',
    TC_OUTSIDE: 'Outside',
}

TRIGGER_KINDS = {
    TK_UNKNOWN: 'Unknown',
    TK_RISINGEDGE: 'Rising edge',
    TK_FALLINGEDGE: 'Falling edge',
    TK_INWINDOW: 'In window',
    TK_OUTWINDOW: 'Out window',
    TK_ANYEDGE: 'Any edge',
    TK_ENTERWINDOW: 'Enter window',
    TK_EXITWINDOW: 'Exit window',
    TK_PULSEWIDTHPOSITIVE: 'Pulse width positive',
    TK_PULSEWIDTHNEGATIVE: 'Pulse width negative',
    TK_PULSEWIDTHEITHER: 'Pulse width either',
    TK_RUNTPULSEPOSITIVE: 'Runt pulse positive',
    TK_RUNTPULSENEGATIVE: 'Runt pulse negative',
    TK_RUNTPULSEEITHER: 'Runt pulse either',
    TK_INTERVALRISING: 'Interval rising',
    TK_INTERVALFALLING: 'Interval falling',
}

TRIGGER_LEVEL_MODES = {
    TLM_UNKNOWN: 'Unknown',
    TLM_RELATIVE: 'Relative',
    TLM_ABSOLUTE: 'Absolute',
}

TRIGGER_OUTPUT_EVENTS = {
    TOE_UNKNOWN: 'Unknown',
    TOE_GENERATOR_START: 'Generator start',
    TOE_GENERATOR_STOP: 'Generator stop',
    TOE_GENERATOR_NEWPERIOD: 'Generator new period',
    TOE_OSCILLOSCOPE_RUNNING: 'Oscilloscope running',
    TOE_OSCILLOSCOPE_TRIGGERED: 'Oscilloscope triggered',
    TOE_MANUAL: 'Manual',
}



from ctypes import *
from .api import api
from .const import *
from .library import library


class TriggerOutput(object):
    """"""

    def __init__(self, handle, index):
        self._handle = handle
        self._index = index

    def _get_enabled(self):
        """ Check whether a trigger output is enabled. """
        value = api.DevTrOutGetEnabled(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_enabled(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.DevTrOutSetEnabled(self._handle, self._index, value)
        library.check_last_status_raise_on_error()

    def _get_events(self):
        """ Supported trigger output events. """
        value = api.DevTrOutGetEvents(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value

    def _get_event(self):
        """ Currently selected trigger output event. """
        value = api.DevTrOutGetEvent(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value

    def _set_event(self, value):
        api.DevTrOutSetEvent(self._handle, self._index, value)
        library.check_last_status_raise_on_error()

    def _get_id(self):
        """ Id. """
        value = api.DevTrOutGetId(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value

    def _get_name(self):
        """ Name. """
        length = api.DevTrOutGetName(self._handle, self._index, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.DevTrOutGetName(self._handle, self._index, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def trigger(self):
        """ Trigger the specified device trigger output.

        :returns: ``True`` if successful, ``False`` otherwise.
        .. versionadded:: 0.6
        """
        result = api.DevTrOutTrigger(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    enabled = property(_get_enabled, _set_enabled)
    events = property(_get_events)
    event = property(_get_event, _set_event)
    id = property(_get_id)
    name = property(_get_name)



class ObjectList(object):
    """"""

    def __init__(self):
        self._items = []

    def __getitem__(self, index):
        if index >= 0 and index < len(self._items):
            return self._items[index]
        else:
            raise IndexError()

    def __len__(self):
        return len(self._items)

    def _get_count(self):
        return len(self._items)

    count = property(_get_count)



import datetime
import socket
import struct
from .const import *
from .types import *


def convert_version(value):
    """Convert UInt64/c_uint64 to Version object."""
    if value == 0:
        return None
    return Version(value >> 48, (value >> 32) & 0xffff, (value >> 16) & 0xffff, value & 0xffff)


def convert_date(value):
    """Convert UInt32/c_uint32 to datetime.date object."""
    if value == 0:
        return None
    return datetime.date(value >> 16, (value >> 8) & 0xff, value & 0xff)


def convert_tristate(value):
    if value == TRISTATE_TRUE:
        return Tristate(True)
    elif value == TRISTATE_FALSE:
        return Tristate(False)
    else:
        return Tristate(None)


def is_string(obj):
    try:
        return isinstance(obj, basestring)
    except NameError:
        return isinstance(obj, str)


def ipv4_str(value):
    return socket.inet_ntoa(struct.pack("!I", value))


def auto_resolution_mode_str(value):
    result = []
    for i in range(ARN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(AUTO_RESOLUTION_MODES[bit])

    return ', '.join(result)


def coupling_str(value):
    result = []
    for i in range(CKN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(COUPLINGS[bit])

    return ', '.join(result)


def clock_output_str(value):
    result = []
    for i in range(CON_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(CLOCK_OUTPUTS[bit])

    return ', '.join(result)


def clock_source_str(value):
    result = []
    for i in range(CSN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(CLOCK_SOURCES[bit])

    return ', '.join(result)


def connector_type_str(value):
    result = []
    for i in range(CONNECTORTYPE_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(CONNECTOR_TYPES[bit])

    return ', '.join(result)


def device_type_str(value):
    result = []
    for i in range(DEVICETYPE_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(DEVICE_TYPES[bit])

    return ', '.join(result)


def frequency_mode_str(value):
    result = []
    for i in range(FMN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(FREQUENCY_MODES[bit])

    return ', '.join(result)


def generator_mode_str(value):
    result = []
    for i in range(GMN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(GENERATOR_MODES[bit])

    return ', '.join(result)


def generator_status_str(value):
    result = []
    for i in range(GSN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(GENERATOR_STATUSS[bit])

    return ', '.join(result)


def interface_str(value):
    result = []
    for i in range(INTERFACE_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(INTERFACES[bit])

    return ', '.join(result)


def measure_mode_str(value):
    result = []
    for i in range(MMN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(MEASURE_MODES[bit])

    return ', '.join(result)


def server_error_str(value):
    return SERVER_ERRORS[value]


def server_status_str(value):
    return SERVER_STATUSS[value]


def signal_type_str(value):
    result = []
    for i in range(STN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(SIGNAL_TYPES[bit])

    return ', '.join(result)


def trigger_condition_str(value):
    result = []
    for i in range(TCN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(TRIGGER_CONDITIONS[bit])

    return ', '.join(result)


def trigger_kind_str(value):
    result = []
    for i in range(TKN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(TRIGGER_KINDS[bit])

    return ', '.join(result)


def trigger_level_mode_str(value):
    result = []
    for i in range(TLMN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(TRIGGER_LEVEL_MODES[bit])

    return ', '.join(result)


def trigger_output_event_str(value):
    result = []
    for i in range(TOEN_COUNT):
        bit = 1 << i
        if (value & bit) != 0:
            result.append(TRIGGER_OUTPUT_EVENTS[bit])

    return ', '.join(result)



from ctypes import *
from .api import api
from .const import *
from .utils import *
from .library import library
from .object import Object


class Server(Object):
    """"""

    def __init__(self, handle):
        super(Server, self).__init__(handle)

    def __eq__(self, other):
        if type(self) is type(other):
            return self._handle == other._handle
        return NotImplemented

    def __ne__(self, other):
        if type(self) is type(other):
            return not self.__eq__(other)
        return NotImplemented

    def connect(self, asynchronous=False):
        """ Connect to a specified network instrument or instrument server.

        :param asynchronous: Connect asynchronously
        :returns: ``True`` if successful, ``False`` otherwise.
        .. versionadded:: 0.9
        """
        asynchronous = BOOL8_TRUE if asynchronous else BOOL8_FALSE
        result = api.SrvConnect(self._handle, asynchronous)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def disconnect(self, force=False):
        """ Disconnect from a specified network instrument or instrument server.

        :param force: If ``True`` all open devices are closed, if ``False`` remove only succeeds if no devices are open.
        :returns: ``True`` if successful, ``False`` otherwise.
        .. versionadded:: 0.9
        """
        force = BOOL8_TRUE if force else BOOL8_FALSE
        result = api.SrvDisconnect(self._handle, force)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def remove(self, force=False):
        """ Remove a specified specified network instrument or instrument server from the list of servers.

        :param force: If ``True`` all open devices are closed, if ``False`` remove only succeeds if no devices are open.
        :returns: ``True`` if successful, ``False`` otherwise.
        .. versionadded:: 0.9
        """
        force = BOOL8_TRUE if force else BOOL8_FALSE
        result = api.SrvRemove(self._handle, force)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def _get_status(self):
        """ Retrieve the status of a specified network instrument or instrument server. """
        value = api.SrvGetStatus(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_last_error(self):
        """ Last error from a specified network instrument or instrument server. """
        value = api.SrvGetLastError(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_url(self):
        """ URL of the specified network instrument or instrument server. """
        length = api.SrvGetURL(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.SrvGetURL(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_id(self):
        """ Id of the specified network instrument or instrument server. """
        length = api.SrvGetID(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.SrvGetID(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_ipv4_address(self):
        """ IPv4 address of the specified network instrument or instrument server. """
        value = api.SrvGetIPv4Address(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_ip_port(self):
        """ IP port number of the specified network instrument or instrument server. """
        value = api.SrvGetIPPort(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_name(self):
        """ Name of the specified network instrument or instrument server. """
        length = api.SrvGetName(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.SrvGetName(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_description(self):
        """ Description of the specified network instrument or instrument server. """
        length = api.SrvGetDescription(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.SrvGetDescription(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_version(self):
        """ Software version number of the specified network instrument or instrument server. """
        value = api.SrvGetVersion(self._handle)
        library.check_last_status_raise_on_error()
        return convert_version(value)

    def _get_version_extra(self):
        """ Software version postfix of the specified network instrument or instrument server. """
        length = api.SrvGetVersionExtra(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.SrvGetVersionExtra(self._handle, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    status = property(_get_status)
    last_error = property(_get_last_error)
    url = property(_get_url)
    id = property(_get_id)
    ipv4_address = property(_get_ipv4_address)
    ip_port = property(_get_ip_port)
    name = property(_get_name)
    description = property(_get_description)
    version = property(_get_version)
    version_extra = property(_get_version_extra)



from ctypes import *
from .api import api
from .const import *
from .library import library


class TriggerInput(object):
    """"""

    def __init__(self, handle, index):
        self._handle = handle
        self._index = index

    def _get_is_triggered(self):
        """ Check whether the trigger input caused a trigger. """
        value = api.ScpTrInIsTriggered(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_enabled(self):
        """ Check whether a device trigger input is enabled. """
        value = api.DevTrInGetEnabled(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_enabled(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.DevTrInSetEnabled(self._handle, self._index, value)
        library.check_last_status_raise_on_error()

    def _get_kinds(self):
        """ Supported trigger kinds. """
        value = api.DevTrInGetKinds(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value

    def _get_kind(self):
        """ Currently selected trigger kind. """
        value = api.DevTrInGetKind(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value

    def _set_kind(self, value):
        api.DevTrInSetKind(self._handle, self._index, value)
        library.check_last_status_raise_on_error()

    def _get_is_available(self):
        """ Check whether a device trigger input is available. """
        value = api.DevTrInIsAvailable(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_id(self):
        """ Id. """
        value = api.DevTrInGetId(self._handle, self._index)
        library.check_last_status_raise_on_error()
        return value

    def _get_name(self):
        """ Name. """
        length = api.DevTrInGetName(self._handle, self._index, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.DevTrInGetName(self._handle, self._index, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    is_triggered = property(_get_is_triggered)
    enabled = property(_get_enabled, _set_enabled)
    kinds = property(_get_kinds)
    kind = property(_get_kind, _set_kind)
    is_available = property(_get_is_available)
    id = property(_get_id)
    name = property(_get_name)



from ctypes import *
from .api import api
from .const import *
from .library import library
from .networkservers import NetworkServers


class Network(object):
    """"""

    def __init__(self):
        self._servers = NetworkServers()

    def _get_servers(self):
        return self._servers

    def _get_auto_detect_enabled(self):
        """ Check whether automatically detecting network instruments and instrument servers is enabled. """
        value = api.NetGetAutoDetectEnabled()
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_auto_detect_enabled(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.NetSetAutoDetectEnabled(value)
        library.check_last_status_raise_on_error()

    auto_detect_enabled = property(_get_auto_detect_enabled, _set_auto_detect_enabled)
    servers = property(_get_servers)


network = Network()
